function y = nanstd(varargin)
%NANSTD Standard deviation, ignoring NaNs.
%   nanstd is not recommended. Use std instead.
%
%   Y = NANSTD(X) returns the sample standard deviation of the values in X,
%   treating NaNs as missing values.  For a vector input, Y is the standard
%   deviation of the non-NaN elements of X.  For a matrix input, Y is a row
%   vector containing the standard deviation of the non-NaN elements in
%   each column of X. For N-D arrays, NANSTD operates along the first
%   non-singleton dimension of X.
%
%   NANSTD normalizes Y by (N-1), where N is the sample size.  This is the
%   square root of an unbiased estimator of the variance of the population
%   from which X is drawn, as long as X consists of independent, identically
%   distributed samples and data are missing at random.
%
%   Y = NANSTD(X,1) normalizes by N and produces the square root of the
%   second moment of the sample about its mean.  NANSTD(X,0) is the same as
%   NANSTD(X).
%
%   Y = NANSTD(X,FLAG,'all') returns the standard deviation of all elements
%   of X. Pass in 0 for W to use the default normalization by N-1, or 1 to 
%   use N.
%
%   Y = NANSTD(X,FLAG,DIM) takes the standard deviation along the dimension 
%   DIM of X.
%
%   Y = NANSTD(X,FLAG,VECDIM) finds the standard deviation of the elements
%   of X based on the dimensions specified in the vector VECDIM.
%
%   See also STD, NANVAR, NANMEAN, NANMEDIAN, NANMIN, NANMAX, NANSUM.

%   Copyright 1993-2020 The MathWorks, Inc.


% Call nanvar(x,flag,dim) with as many inputs as needed
y = sqrt(nanvar(varargin{:}));
