modules_path = 'modules';
addpath(genpath(modules_path));

a=1; %1-70 Create excel for each simulation in folder /results

init_format = 'EXCEL'; %EXCEL or YAML
run_name = ['test_vegetation_snow_',num2str(a)]; %parameter file name and result directory 
constant_file = 'CONSTANTS_excel'; %file with constants
result_path = 'results/';  %with trailing backslash
forcing_path = fullfile ('./forcing/');

% integer code for soil_type; 1: sand; 2: silt: 3: clay: 4: peat; 5: water (i.e. approximation of free water, very large-pore ground material).

provider = PROVIDER;
provider = assign_paths(provider, init_format, run_name, result_path, constant_file, forcing_path);
provider = read_const(provider);
provider = read_parameters(provider);

% Possibility to overwrite soil parameters here if incorrect in excel sheet
% Chukotka
% provider.CLASSES.STRAT_layers{1, 1}.PARA.layers.depth = [0;0.06;0.18;1;10];
% provider.CLASSES.STRAT_layers{1, 1}.PARA.layers.waterIce = [0.600000000000000;0.600000000000000;0.600000000000000;0.600000000000000;0.600000000000000];
% provider.CLASSES.STRAT_layers{1, 1}.PARA.layers.mineral = [0;0.100000000000000;0.400000000000000;0.400000000000000;0.970000000000000];
% provider.CLASSES.STRAT_layers{1, 1}.PARA.layers.organic = [0.300000000000000;0.300000000000000;0;0;0];
% provider.CLASSES.STRAT_layers{1, 1}.PARA.layers.field_capacity = [0.300000000000000;0.300000000000000;0.300000000000000;0.300000000000000;0.030000000000000];
% provider.CLASSES.STRAT_layers{1, 1}.PARA.layers.soil_type = [4;3;3;3;1];
% provider.CLASSES.VEGETATION_ml_canopy{1, 1}.PARA.ztop = 11;
% provider.CLASSES.VEGETATION_ml_canopy{1, 1}.PARA.zref = 16;
% provider.CLASSES.VEGETATION_ml_canopy{1, 1}.PARA.zref_old = 16;
% provider.CLASSES.FORCING_seb_vegetation{1, 1}.PARA.filename = 'Chukotka_tcc_T0_ERAint_1979_2019_trialSimone.mat';

%create the RUN_INFO class
[run_info, provider] = run_model(provider);

%   rest is equivalent to 
%[run_info, tile] = run(run_info);
%   for RUN_INFO class RUN_1D_STANDARD

%create the TILE
tile = copy(run_info.PPROVIDER.CLASSES.(run_info.PARA.tile_class){run_info.PARA.tile_class_index,1});
tile.RUN_INFO = run_info;
run_info.TILE = tile;

%create the stratigraphy
tile = finalize_init(tile);

%    rest is equivalent to 
%tile = run(tile);
%   for TILE class TILE_1D_STANDARD

TOP_CLASS = tile.TOP_CLASS;
BOTTOM_CLASS = tile.BOTTOM_CLASS;
TOP = tile.TOP;
BOTTOM = tile.BOTTOM;
TOP.LATERAL = tile.LATERAL;

%=========================================================================
%TIME INTEGRATION
%=========================================================================
while tile.t < tile.FORCING.PARA.end_time
        
    %interpolate focing data to time t
    tile = interpolate_forcing_tile(tile);

    %upper boundar condition (uppermost class only)
    TOP.NEXT = get_boundary_condition_u(TOP.NEXT, tile);
    
    %set fluxes between classes in the stratigrapht
    CURRENT = TOP.NEXT;
    while ~isequal(CURRENT.NEXT, BOTTOM)
        get_boundary_condition_m(CURRENT.IA_NEXT, tile); %call interaction class function
        CURRENT = CURRENT.NEXT;
    end

    %lower boundary condition (lowermost class)
    CURRENT = get_boundary_condition_l(CURRENT,  tile);  %At this point, CURRENT is equal to BOTTOM_CLASS

    %calculate spatial derivatives
    CURRENT = TOP.NEXT;
    while ~isequal(CURRENT, BOTTOM)
        CURRENT = get_derivatives_prognostic(CURRENT, tile);
        CURRENT = CURRENT.NEXT;
    end
    
    %calculate timestep [second]
    CURRENT = TOP.NEXT;
    tile.timestep = 1e8;
    while ~isequal(CURRENT, BOTTOM)
        tile.timestep = min(tile.timestep, get_timestep(CURRENT, tile));
        CURRENT = CURRENT.NEXT;
    end
    %tile.timestep = min(4, tile.timestep);
    tile.next_break_time = min(tile.LATERAL.IA_TIME, tile.OUT.OUTPUT_TIME);
    tile.timestep = min(tile.timestep, (tile.next_break_time - tile.t).*tile.CONST.day_sec);
    
    %prognostic step - integrate prognostic variables in time
    CURRENT = TOP.NEXT;
    while ~isequal(CURRENT, BOTTOM)
        CURRENT = advance_prognostic(CURRENT, tile);
        CURRENT = CURRENT.NEXT;
    end
%     disp(TOP_CLASS.TEMP.d_water(1,1))
%test=[test; [TOP_CLASS.STATVAR.T(1,1) TOP_CLASS.STATVAR.waterIce(1,1) TOP_CLASS.STATVAR.waterPotential(1,1) TOP_CLASS.TEMP.d_water(1,1)] tile.timestep];
           
    %diagnostic step - compute diagnostic variables
    TOP.NEXT = compute_diagnostic_first_cell(TOP.NEXT, tile); %calculate Lstar, only uppermost class
    CURRENT = BOTTOM.PREVIOUS;
    while ~isequal(CURRENT, TOP)
        CURRENT = compute_diagnostic(CURRENT, tile);
        CURRENT = CURRENT.PREVIOUS;
    end
    
    %triggers
    CURRENT = TOP.NEXT;
    while ~isequal(CURRENT, BOTTOM)
        CURRENT = check_trigger(CURRENT, tile);
        CURRENT = CURRENT.NEXT;
    end
    
    tile = interact_lateral(tile);
    
    %set TOP_CLASS and BOTTOM_CLASS for convenient access
    TOP_CLASS = TOP.NEXT; 
    BOTTOM_CLASS = BOTTOM.PREVIOUS;
    
    

    %update time variable t
    tile.t = tile.t + tile.timestep./tile.CONST.day_sec;
    
    %model 
    tile = store_OUT_tile(tile);
   
end
% end

% for a=25:27
 [a] = write_out_table(a);
% end
% end
% end