function [a] = write_out_table(a)
    
%results are stored in results/"run_number"
%this folder must contain an Excel spreadsheet called "run_number".xlsx deifining the properties of all classes requried for the run and the
%spreadsheet "CONSTANTS_excel.xlsx" containing all constants
modules_path = 'modules';
addpath(genpath(modules_path));

init_format = 'EXCEL'; %EXCEL or YAML
run_name = ['test_vegetation_snow_',num2str(a)]; %parameter file name and result directory 
%run_name = 'ExperimentHansen2004';
constant_file = 'CONSTANTS_excel'; %file with constants
result_path = 'results/';  %with trailing backslash
forcing_path = fullfile ('./forcing/');

%read information for all classes from Excel file and store it in cell
%arrays
provider = PROVIDER;
provider = assign_paths(provider, init_format, run_name, result_path, constant_file, forcing_path);
provider = read_const(provider);
provider = read_parameters(provider);

%create the RUN_INFO class
[run_info, provider] = run_model(provider);

%   rest is equivalent to 
%[run_info, tile] = run(run_info);
%   for RUN_INFO class RUN_1D_STANDARD

%create the TILE
tile = copy(run_info.PPROVIDER.CLASSES.(run_info.PARA.tile_class){run_info.PARA.tile_class_index,1});
tile.RUN_INFO = run_info;
run_info.TILE = tile;

%create the stratigraphy
tile = finalize_init(tile);

%    rest is equivalent to 
%tile = run(tile);
%   for TILE class TILE_1D_STANDARD

% TOP_CLASS = tile.TOP_CLASS;
% BOTTOM_CLASS = tile.BOTTOM_CLASS;
% TOP = tile.TOP;
% BOTTOM = tile.BOTTOM;
% TOP.LATERAL = tile.LATERAL;

% day_sec = 24.*3600;
t = tile.FORCING.PARA.end_time;

load([result_path run_name '/' run_name '_' datestr(t,'yyyymmdd') '.mat']);

[meanW_snowCovered, stdW_snowCovered, minW_snowCovered, maxW_snowCovered, meanW_snowFree, stdW_snowFree, minW_snowFree, maxW_snowFree, maxW, stdW, meanW, minW, maxSnow, maxALT, meanALT, meanT, maxT, minT, meanT_snowCovered, minT_snowCovered, maxT_snowCovered, meanT_snowFree, minT_snowFree, maxT_snowFree, stdT_snowCovered, stdT_snowFree, stdT, PlantWater_av] = read_out_Spasskaya(run_name, out);
% [meanW_snowCovered, stdW_snowCovered, minW_snowCovered, maxW_snowCovered, meanW_snowFree, stdW_snowFree, minW_snowFree, maxW_snowFree, maxW, stdW, meanW, minW, maxSnow, maxALT, meanALT, meanT, maxT, minT, meanT_snowCovered, minT_snowCovered, maxT_snowCovered, meanT_snowFree, minT_snowFree, maxT_snowFree, stdT_snowCovered, stdT_snowFree, stdT] = read_out_Spasskaya(run_name, out);

Parameters_forest = [meanW_snowCovered, stdW_snowCovered, minW_snowCovered, maxW_snowCovered, meanW_snowFree, stdW_snowFree, minW_snowFree, maxW_snowFree, maxW, stdW, meanW, minW, maxSnow, maxALT, meanALT, meanT, maxT, minT, meanT_snowCovered, minT_snowCovered, maxT_snowCovered, meanT_snowFree, minT_snowFree, maxT_snowFree, stdT_snowCovered, stdT_snowFree, stdT, PlantWater_av];
Parameters_forest_names = {'meanW_snowCovered', 'stdW_snowCovered', 'minW_snowCovered', 'maxW_snowCovered', 'meanW_snowFree', 'stdW_snowFree', 'minW_snowFree', 'maxW_snowFree', 'maxW', 'stdW', 'meanW', 'minW', 'maxSnow', 'maxALT', 'meanALT', 'meanT', 'maxT', 'minT', 'meanT_snowCovered', 'minT_snowCovered', 'maxT_snowCovered', 'meanT_snowFree', 'minT_snowFree', 'maxT_snowFree', 'stdT_snowCovered', 'stdT_snowFree', 'stdT', 'PlantWater_av'};
Parameters_forest = array2table(Parameters_forest,'VariableNames',Parameters_forest_names);

writetable(Parameters_forest,[run_name '_Parameters_Chukotka.xlsx'],'WriteRowNames',true);
end

